/*        LTR34    .
              
    (   f_send_data())
         DAC_CHANNEL_CNT.
          .   
         .

            ,    
           :
    ltr34_ring  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (     )
     :    ,       

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr34api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .
*/


#include "ltr/include/ltr34api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

#ifndef M_PI
    #define M_PI   3.14159265358979323846   /* pi */
#endif

/*      */
#define DAC_CH_SIGNAL_SIZE   500
/*    */
#define DAC_CHANNEL_CNT        1


#ifndef _WIN32
/*      */
static int f_out = 0;


/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}


static INT f_send_data(TLTR34 *hnd, unsigned ch_size) {

    DWORD snd_size = hnd->ChannelQnt*ch_size;
    INT err=LTR_OK;
    unsigned ch, i;

    double *data = malloc(snd_size*sizeof(data[0]));
    DWORD  *wrds = malloc(snd_size*sizeof(wrds[0]));

    if ((data == NULL) || (wrds == NULL)) {
        err = LTR_ERROR_MEMORY_ALLOC;
    } else {

        for (ch=0; ch < hnd->ChannelQnt; ch++) {
            for (i=0; i < ch_size; i++) {
                /*       ,  
                   ( )  */
                if (ch==0) {
                    data[hnd->ChannelQnt*i + ch] = 10.*sin(2.*M_PI*i/ch_size);
                } else {
                    data[hnd->ChannelQnt*i + ch] = 10.*i/ch_size;
                }
            }

        }

        err = LTR34_ProcessData(hnd, data, wrds, snd_size, TRUE);
        if (err!=LTR_OK) {
            fprintf(stderr, "   (%d) : %s!\n", err, LTR34_GetErrorString(err));
        }


        if (err == LTR_OK) {
            INT sent_size = LTR34_Send(hnd, wrds, snd_size, 5000);
            if (sent_size < 0) {
                err = sent_size;
                fprintf(stderr, "   (%d) : %s!\n", err, LTR34_GetErrorString(err));
            } else if (sent_size != (INT)snd_size){
                fprintf(stderr, "!      ( %d,  %d)\n",
                        snd_size, sent_size);
                err = LTR_ERROR_SEND_INSUFFICIENT_DATA;
            }
        }
    }

    free(data);
    free(wrds);
    return err;
}



int main(int argc, char** argv) {
    INT err = LTR_OK;
    TLTR34 hltr34;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*      ... */
    setlocale(LC_ALL, "");
#endif


    err = f_get_params(argc, argv, &par);

    LTR34_Init(&hltr34);

    err = LTR34_Open(&hltr34, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
    if (err != LTR_OK) {
        fprintf(stderr, "     .  %d: %s\n",
                err, LTR34_GetErrorString(err));
    } else {
        int ch = 0;
        printf("  .   :\n   %s\n   %s\n    %s\n     - %d\n",
               hltr34.ModuleInfo.Name, hltr34.ModuleInfo.Serial, hltr34.ModuleInfo.FPGA_Version, hltr34.ModuleInfo.MaxChannelQnt);
        fflush(stdout);

        hltr34.ChannelQnt = DAC_CHANNEL_CNT;
        for (ch=0; ch < hltr34.ChannelQnt; ch++) {
            hltr34.LChTbl[ch] = LTR34_CreateLChannel(ch + 1, 0);
        }
        hltr34.FrequencyDivisor = 60;
        hltr34.AcknowledgeType = LTR34_ACKTYPE_STATUS;
        hltr34.RingMode = TRUE;
        hltr34.UseClb = TRUE;
        hltr34.ExternalStart = FALSE;

        err = LTR34_Reset(&hltr34);
        if (err != LTR_OK) {
            fprintf(stderr, "     LTR34.  %d: %s\n",
                    err, LTR34_GetErrorString(err));
        } else {
            err = LTR34_Config(&hltr34);
            if (err != LTR_OK) {
                fprintf(stderr, "    LTR34.  %d: %s\n",
                        err, LTR34_GetErrorString(err));
            }
        }

        /*       */
        if (err == LTR_OK)
            err = f_send_data(&hltr34, DAC_CH_SIGNAL_SIZE);


        if (err == LTR_OK) {
            INT stop_err;

            err = LTR34_DACStart(&hltr34);
            if (err != LTR_OK) {
                fprintf(stderr, "    .  %d: %s\n",
                        err, LTR34_GetErrorString(err));
            }

            if (err == LTR_OK) {
                printf("  .    %s\n",
#ifdef _WIN32
                       " "
#else
                       "CTRL+C"
#endif
                       );
                fflush(stdout);

#ifdef _WIN32
                /*    */
                _getch();
#else
                while (!f_out)
                    pause();
#endif

            }

            /*    */
            printf(" ...\n");
            fflush(stdout);
            stop_err = LTR34_DACStop(&hltr34);
            if (stop_err == LTR_OK) {
                printf("   !\n");
                fflush(stdout);
            } else {
                fprintf(stderr, "    .  %d: %s\n",
                        err, LTR34_GetErrorString(err));
                if (err == LTR_OK)
                    err = stop_err;
            }
        }


        LTR34_Close(&hltr34);
    }
    return err;
}
