/*        LTR34   .
        .
         DAC_CHANNEL_CNT,  
       sig_amps (  )  sig_freq (  ).
             
             
      .

            ,    
           :
    ltr34_stream_sin  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (     )
     :    ,       

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr34api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .
*/

#include "ltr/include/ltr34api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

#ifndef M_PI
    #define M_PI   3.14159265358979323846   /* pi */
#endif


#define DAC_BLOCK_SEND_SIZE   5000
#define DAC_PREPARE_SIZE     40000

/*    */
#define DAC_CHANNEL_CNT         1

/*    (   DAC_CHANNEL_CNT ) */
static const double sig_amps[LTR34_DAC_NUMBER_MAX]   = {10.,      2.,   0.,   0.,   0.,   0.,   0.,   0.};
static const double sig_freq[LTR34_DAC_NUMBER_MAX]   = {4000.1, 4000, 4000, 4000, 4000, 4000, 4000, 4000};




/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif



typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}

/*    ch_size      .
 *  ch_cur_phases        ,
 *     */
static INT f_send_data(TLTR34 *hnd, unsigned ch_size, double *ch_cur_phases) {

    DWORD snd_size = hnd->ChannelQnt*ch_size;
    INT err=LTR_OK;
    unsigned ch, i;

    double *data = malloc(snd_size*sizeof(data[0]));
    DWORD  *wrds = malloc(snd_size*sizeof(wrds[0]));

    if ((data == NULL) || (wrds == NULL)) {
        err = LTR_ERROR_MEMORY_ALLOC;
    } else {
        /*          */
        for (ch=0; ch < hnd->ChannelQnt; ch++) {
            for (i=0; i < ch_size; i++) {
                data[hnd->ChannelQnt*i + ch] = sig_amps[ch]*sin(ch_cur_phases[ch]);
                ch_cur_phases[ch] += 2*M_PI *sig_freq[ch]/hnd->FrequencyDAC;
            }

        }

        err = LTR34_ProcessData(hnd, data, wrds, snd_size, TRUE);
        if (err!=LTR_OK) {
            fprintf(stderr, "   (%d) : %s!\n", err, LTR34_GetErrorString(err));
        }


        if (err == LTR_OK) {
            INT sent_size = 0;

            /*  ,      ,   
             *  ,     .
             *         ,   
             *     .   
             *   Send   
             */
            while (!f_out && (err == LTR_OK) && (sent_size < (INT)snd_size)) {
                INT cur_sent = LTR34_Send(hnd, &wrds[sent_size], snd_size-sent_size, 100);
                if (cur_sent < 0) {
                    err = cur_sent;
                    fprintf(stderr, "   (%d) : %s!\n", err, LTR34_GetErrorString(err));
                } else {
                    sent_size += cur_sent;
                }
#ifdef _WIN32
                /*      */
                if (err==LTR_OK) {
                    if (_kbhit())
                        f_out = 1;
                }
#endif
            }
        }
    }
    free(data);
    free(wrds);
    return err;
}



int main(int argc, char** argv) {
    INT err = LTR_OK;
    TLTR34 hltr34;
    t_open_param par;
    /*   ,      */
    double ch_cur_phases[LTR34_DAC_NUMBER_MAX]    = {0,0,0,0,0,0,0,0};

#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*      ... */
    setlocale(LC_ALL, "");
#endif


    err = f_get_params(argc, argv, &par);

    LTR34_Init(&hltr34);

    err = LTR34_Open(&hltr34, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
    if (err != LTR_OK) {
        fprintf(stderr, "     .  %d: %s\n",
                err, LTR34_GetErrorString(err));
    } else {
        int ch = 0;
        printf("  .   :\n   %s\n   %s\n    %s\n     - %d\n",
               hltr34.ModuleInfo.Name, hltr34.ModuleInfo.Serial, hltr34.ModuleInfo.FPGA_Version, hltr34.ModuleInfo.MaxChannelQnt);
        fflush(stdout);

        hltr34.ChannelQnt = DAC_CHANNEL_CNT;
        for (ch=0; ch < hltr34.ChannelQnt; ch++) {
            hltr34.LChTbl[ch] = LTR34_CreateLChannel(ch + 1, 0);
        }
        hltr34.FrequencyDivisor = 60;
        hltr34.AcknowledgeType = LTR34_ACKTYPE_STATUS;
        hltr34.RingMode = FALSE;
        hltr34.UseClb = TRUE;
        hltr34.ExternalStart = FALSE;

        err = LTR34_Reset(&hltr34);
        if (err != LTR_OK) {
            fprintf(stderr, "     LTR34.  %d: %s\n",
                    err, LTR34_GetErrorString(err));
        } else {
            err = LTR34_Config(&hltr34);
            if (err != LTR_OK) {
                fprintf(stderr, "    LTR34.  %d: %s\n",
                        err, LTR34_GetErrorString(err));
            }
        }

        /*      ,    
                  */
        if (err == LTR_OK)
            err = f_send_data(&hltr34, DAC_PREPARE_SIZE, ch_cur_phases);


        if (err == LTR_OK) {
            INT stop_err;
            /*   */
            err = LTR34_DACStart(&hltr34);
            if (err != LTR_OK) {
                fprintf(stderr, "    .  %d: %s\n",
                        err, LTR34_GetErrorString(err));
            }

            if (err == LTR_OK) {
                printf("  .    %s\n",
#ifdef _WIN32
                       " "
#else
                       "CTRL+C"
#endif
                       );
                fflush(stdout);

                /*        
                 *    */
                while (!f_out && (err == LTR_OK)) {
                    err = f_send_data(&hltr34, DAC_BLOCK_SEND_SIZE, ch_cur_phases);
                }
            }


            /*    */
            printf(" ...\n");
            fflush(stdout);
            stop_err = LTR34_DACStop(&hltr34);
            if (stop_err == LTR_OK) {
                printf("   !\n");
                fflush(stdout);
            } else {
                fprintf(stderr, "    .  %d: %s\n",
                        err, LTR34_GetErrorString(err));
                if (err == LTR_OK)
                    err = stop_err;
            }
        }


        LTR34_Close(&hltr34);
    }
    return err;
}
